NOTE: Underwear is under heavy and frequent development and is not recommeded for use yet.

Underwear is based on Suspenders, the base Rails application used at
[thoughtbot](http://thoughtbot.com).

## Installation

First install the underwear gem:

    gem install underwear

Then run:

    underwear projectname

This will create a Rails app in `projectname` using the latest version of Rails.
Note: This will also create Heroku staging and production apps using the name you provide, so make it unique.

## Gemfile

To see the latest and greatest gems, look at Underwear'
[Gemfile](templates/Gemfile.erb), which will be appended to the default
generated projectname/Gemfile.

It includes application gems like:

* [AASM](https://github.com/aasm/aasm) for state machines for Ruby classes
* [Autoprefixer Rails](https://github.com/ai/autoprefixer-rails) for CSS vendor prefixes
* [Awesome Print](https://github.com/michaeldv/awesome_print) Pretty print your Ruby objects with style
* [Devise](https://github.com/plataformatec/devise) Flexible authentication solution.
* [FriendlyId](https://github.com/norman/friendly_id) FriendlyId is the "Swiss Army bulldozer" of slugging and permalink plugins for Active Record.
* [jQuery Rails](https://github.com/rails/jquery-rails) for jQuery
* [New Relic RPM](https://github.com/newrelic/rpm) for monitoring performance
* [Paperclip](https://github.com/thoughtbot/paperclip) Easy file attachment management for ActiveRecord.
* [Postgres](https://github.com/ged/ruby-pg) for access to the Postgres database
* [Rack Canonical Host](https://github.com/tylerhunt/rack-canonical-host) to
  ensure all requests are served from the same domain
* [Rack Timeout](https://github.com/kch/rack-timeout) to abort requests that are
  taking too long
* [Raven (Sentry)](https://github.com/getsentry/raven-ruby) for exception notification
* [Recipient Interceptor](https://github.com/croaky/recipient_interceptor) to
  avoid accidentally sending emails to real people from staging
* [Semantic UI](https://github.com/doabit/semantic-ui-sass) UI component framework based around useful principles from natural language.
* [Sidekiq](https://github.com/mperham/sidekiq) for background processing
* [Puma](https://github.com/puma/puma) to serve HTTP requests

And development gems like:

* [Bullet](https://github.com/flyerhzm/bullet) help to kill N+1 queries and unused eager loading
* [Bundler Audit](https://github.com/rubysec/bundler-audit) for scanning the
  Gemfile for insecure dependencies based on published CVEs
* [ByeBug](https://github.com/deivid-rodriguez/byebug) for interactively
  debugging behavior
* [Dotenv](https://github.com/bkeepers/dotenv) for loading environment variables
* [Faker](https://github.com/stympy/faker) A library for generating fake data such as names, addresses, and phone numbers.
* [Pry Rails](https://github.com/rweng/pry-rails) for interactively exploring
  objects
* [Spring](https://github.com/rails/spring) for fast Rails actions via
  pre-loading
* [Web Console](https://github.com/rails/web-console) for better debugging via
  in-browser IRB consoles.

And testing gems like:

* [Capybara](https://github.com/jnicklas/capybara) and
  [Capybara Webkit](https://github.com/thoughtbot/capybara-webkit) for
  integration testing
* [Factory Girl](https://github.com/thoughtbot/factory_girl) for test data
* [RSpec Rails](https://github.com/rspec/rspec-rails) RSpec for Rails-3+
* [RSpec Mocks](https://github.com/rspec/rspec-mocks) for stubbing and spying
* [Shoulda Matchers](https://github.com/thoughtbot/shoulda-matchers) for common
  RSpec matchers

TODO
* [Database Cleaner](https://github.com/DatabaseCleaner/database_cleaner) Strategies for cleaning databases in Ruby.
* [Webmock](https://github.com/bblimke/webmock) Library for stubbing and setting expectations on HTTP requests in Ruby.
* [VCR](https://github.com/vcr/vcr) Record your test suite's HTTP interactions and replay them during future test runs for fast, deterministic, accurate tests.
* [Simplecov](https://github.com/colszowka/simplecov) Code coverage for Ruby 1.9+ with a powerful configuration library and automatic merging of coverage across test suites.


## Other goodies

Underwear also comes with:

* The [`./bin/setup`][setup] convention for new developer setup
* The `./bin/deploy` convention for deploying to Heroku
* Rails' flashes set up and in application layout
* A few nice time formats set up for localization
* `Rack::Deflater` to [compress responses with Gzip][compress]
* A [low database connection pool limit][pool]
* [Safe binstubs][binstub]
* [t() and l() in specs without prefixing with I18n][i18n]
* An automatically-created `SECRET_KEY_BASE` environment variable in all
  environments
* Configuration for [CircleCI][circle] Continuous Integration (tests)
* The analytics adapter [Segment][segment] (and therefore config for Google
  Analytics, Intercom, Facebook Ads, Twitter Ads, etc.)

[setup]: http://robots.thoughtbot.com/bin-setup
[compress]: http://robots.thoughtbot.com/content-compression-with-rack-deflater/
[pool]: https://devcenter.heroku.com/articles/concurrency-and-database-connections
[binstub]: https://github.com/thoughtbot/underwear/pull/282
[i18n]: https://github.com/thoughtbot/underwear/pull/304
[circle]: https://circleci.com/docs
[segment]: https://segment.com

## Heroku

You can optionally create Heroku staging and production apps:

    underwear app --heroku true

This:

* Creates a staging and production Heroku app
* Sets them as `staging` and `production` Git remotes
* Configures staging with `RACK_ENV` and `RAILS_ENV` environment variables set
  to `staging`
* Adds the [Rails Stdout Logging][logging-gem] gem
  to configure the app to log to standard out,
  which is how [Heroku's logging][heroku-logging] works.

[logging-gem]: https://github.com/heroku/rails_stdout_logging
[heroku-logging]: https://devcenter.heroku.com/articles/logging#writing-to-your-log

You can optionally specify alternate Heroku flags:

    underwear app \
      --heroku true \
      --heroku-flags "--region eu --addons newrelic,sendgrid,ssl"

See all possible Heroku flags:

    heroku help create

## Git

This will initialize a new git repository for your Rails app. You can
bypass this with the `--skip-git` option:

    underwear app --skip-git true

## GitHub

You can optionally create a GitHub repository for the suspended Rails app. It
requires that you have [Hub](https://github.com/github/hub) on your system:

    curl http://hub.github.com/standalone -sLo ~/bin/hub && chmod +x ~/bin/hub
    underwear app --github organization/project

This has the same effect as running:

    hub create organization/project

## Spring

Underwear uses [spring](https://github.com/rails/spring) by default.
It makes Rails applications load faster, but it might introduce confusing issues
around stale code not being refreshed.
If you think your application is running old code, run `spring stop`.
And if you'd rather not use spring, add `DISABLE_SPRING=1` to your login file.

## Dependencies

Underwear requires the latest version of Ruby.

Some gems included in Underwear have native extensions. You should have GCC
installed on your machine before generating an app with Underwear.

Use [OS X GCC Installer](https://github.com/kennethreitz/osx-gcc-installer/) for
Snow Leopard (OS X 10.6).

Use [Command Line Tools for XCode](https://developer.apple.com/downloads/index.action)
for Lion (OS X 10.7) or Mountain Lion (OS X 10.8).

We use [Capybara Webkit](https://github.com/thoughtbot/capybara-webkit) for
full-stack JavaScript integration testing. It requires QT. Instructions for
installing QT are
[here](https://github.com/thoughtbot/capybara-webkit/wiki/Installing-Qt-and-compiling-capybara-webkit).

PostgreSQL needs to be installed and running for the `db:create` rake task.

## Issues

If you have problems, please create a
[GitHub Issue](https://github.com/thoughtbot/underwear/issues).

## Contributing

See [CONTRIBUTING.md](CONTRIBUTING.md).

## License

[LICENSE]: LICENSE
